package mocman.interpreter.syntax;

import mocman.interpreter.syntax.token.*;
import mocman.util.*;
import java.util.*;

public class Rules
{
	private static Vector<Pair<Integer,Vector<Integer>>> rules= new Vector<Pair<Integer,Vector<Integer>>>();
	private static String starting;
	public static void setStartingRule(String s)
	{
		starting=s;
	}
	public static int getRuleCount()
	{
		return rules.size();
	}
	public static void addRule(int i,int[] t)
	{
		Vector<Integer> p = new Vector<Integer>();
		for (int j=0;j<t.length;j++)
		{
			p.addElement(t[j]);
		}
		rules.addElement(new Pair<Integer,Vector<Integer>>(new Integer(i),p));
	}
	public static String getRight(int ruleindex)
	{
		if (ruleindex<0) return "";
		Pair<Integer,Vector<Integer>> p=getRule(ruleindex);		
		StringBuilder s=new StringBuilder();
		for (int i=0;i<p.getValue().size();i++)
		{
			//s.append(i);
			int k=p.getValue().get(i);
			s.append(Tokenizer.getTokenName(k));
			s.append("&nbsp;");
		}
		return s.toString();

		
	}
	public static int getStartRule()
	{
		for (int i=0;i<rules.size();i++)
		{
			if (rules.get(i).getKey()==100) return i;
		}
		return -1;
	}
	public static int getHash()
	{
		return rules.get(getStartRule()).getValue().get(1);
	}
	public static boolean isTerminal(int i)
	{
		return i<100;
	}
	public static boolean hasEpsilonRule(int tokenid)
	{
		for (int i=0;i<rules.size();i++)
		{
			Pair<Integer,Vector<Integer>> p=getRule(i);
			if (p.getKey()==tokenid)
			{
				if(p.getValue().size()==1)
				{
					if (p.getValue().get(0)==Tokenizer.epsilon())
					{
						return true;
					}
				}
			}
		}
		return false;
	}

	public static void makeTable()
	{
		PassMatrix pmF = new PassMatrix();
		PassMatrix pmB = new PassMatrix();
		PassMatrix pmL = new PassMatrix();

		for (int i=0;i<rules.size();i++)
		{
			Pair<Integer,Vector<Integer>> p=getRule(i);
			//L
			if (p.getValue().size()>0)
			{
				int n = p.getValue().size()-1;
				int Xn = p.getValue().get(n);
				pmL.set(Xn,p.getKey());
				while((hasEpsilonRule(Xn))&&(n>0))
				{
					n--;
					Xn = p.getValue().get(n);
					pmL.set(Xn,p.getKey());
				}
			}
			//B
			for (int j=0;j<p.getValue().size()-1;j++)
			{
				int X1 = p.getValue().get(j);
				int X2 = p.getValue().get(j+1);
				pmB.set(X1,X2);
				int k=2;
				while((j+k<p.getValue().size())&&(hasEpsilonRule(X2)))
				{
					X2= p.getValue().get(j+k);
					pmB.set(X1,X2);
					k++;
				}
			}
			//F
			if (p.getValue()!=null)
			{
				int j=0;
				boolean b=true;
				while ((j<p.getValue().size())&&(b))
				{
					int l=p.getValue().get(j);
					pmF.set(p.getKey(),l);
					if (hasEpsilonRule(l))
					{
						j++;
					}
					else
					{
						b=false;
					}
				}
			}
		}
		pmF.refl();
		//pmB.refl();
		pmL.refl();
		pmF.closure();
		pmL.closure();
		PassMatrix pmFollow = new PassMatrix();
		pmFollow = pmL.mul(pmB).mul(pmF);
		for (int i=0;i<rules.size();i++)
		{
			Pair<Integer,Vector<Integer>> p=getRule(i);
			int X = Tokenizer.getPatternId(p.getKey());
			if ((p.getValue()!=null)&&(p.getValue().size()>0))
			{
				int Beta=p.getValue().get(0);
				int beta=Tokenizer.getPatternId(Beta);
				for (int b=0;b<Tokenizer.getPatternCount();b++)
				{
					if (isTerminal(Tokenizer.getPattern(b).getType()))
					{
//						if ((pmF.get(beta,b)==1)||((hasEpsilonRule(beta))&&(pmFollow.get(X,b)==1)))
						if ((pmF.get(beta,b)==1)||((Beta==Tokenizer.epsilon()&&(pmFollow.get(X,b)==1))))
						{
							ParserTable.insert(p.getKey(),Tokenizer.getPattern(b).getType(),i);
						}
					}
				}
			}
		}
	}

	public static String getRuleName(int ruleindex)
	{
		Pair<Integer,Vector<Integer>> p=getRule(ruleindex);		
		StringBuilder s=new StringBuilder();
		s.append("&lt;");
		s.append(Tokenizer.getTokenName(p.getKey()));
		s.append("&gt; -&gt; ");
		s.append(getRight(ruleindex));
		return s.toString();
	}

	public static Pair<Integer,Vector<Integer>> getRule(int index)
	{
		if (index<0) return null;
		return rules.get(index);
	}
}

