package mocman.util.data;

import java.util.*;

/**
 * Generic Graph type
 * It contains generic Nodes (Node<TN>),
 * generic Edges (Node<TE>)
 * and the graph representation.
 * It is designed to be shown on an awt from.
 * This class is used later by the GraphPrinter class.
 * @author <a href="mailto:mocman@inf.elte.hu">Tibor Zoltan Varkonyi</a>
 * @param TN value type of the nodes
 * @param TE value type of the edges
 */
public class Graph <TN,TE>
{

	private Vector<Node<TN>> node;
	private Vector<Node<TE>> edge;
	private Vector<Vector<Integer>> phi;
	private int edgecount;
	private int nodecount;

/** Default constructor */
	public Graph()
	{
		edgecount=0;
		nodecount=0;
		node = new Vector<Node<TN>>();
		edge = new Vector<Node<TE>>();
		phi = new Vector<Vector<Integer>>();
	}

/** Gets the node count */
	public int nodeCount()
	{
		return nodecount;
	}

/** Gets the edge count */
	public int edgeCount()
	{
		return edgecount;
	}

/** Gets a node */
	public Node<TN> nodes(int i)
	{
		return node.get(i);
	}

/** Gets an edge */
	public Node<TE> edges(int i)
	{
		return edge.get(i);
	}

/** Adds a node */
	public void add(Node<TN> n)
	{
		node.setSize(nodecount+1);
		node.set(nodecount++,n);
	}

/** Checks if there is an edge between the node indexes.
 *  Use this for directed graphs
 */
	public boolean isEdge(int n1,int n2)
	{
		try
		{
			if (phi.get(n1)==null) return false;
			if (phi.get(n1).get(n2)==null) return false;
			return true;
		}
		catch (Exception e)
		{
			return false;
		}
	}

/** Checks if there is an edge between the node indexes.
 *  Use this for undirected graphs
 */
	public boolean isTwoWayEdge(int n1,int n2)
	{
		return isEdge(n1,n2)||isEdge(n2,n1);
	}

/** Gets the edge between the node indexes.
 *  Use this for directed graphs
 */
	public Node<TE> getEdge(int n1,int n2)
	{
		try
		{
			if (phi.get(n1)==null) return null;
			if (phi.get(n1).get(n2)==null) return null;
			return edge.get(phi.get(n1).get(n2));
		}
		catch (Exception e)
		{
			return null;
		}		
	}

/** Gets the edge between the node indexes.
 *  Use this for undirected graphs
 */
	public Node<TE> getTwoWayEdge(int n1,int n2)
	{
		if (isEdge(n1,n2)) return getEdge(n1,n2);
		if (isEdge(n2,n1)) return getEdge(n2,n1);
		return null;
	}

/** Adds a new edge between the given nodes */
	public void addEdge(int n1,int n2,Node<TE> n)
	{
		edge.setSize(edgecount+1);
		if (phi.size()<=n1)
		{
			phi.setSize(n1+1);
		}
		Vector<Integer> v =phi.get(n1);
		if (v==null) 
		{
			phi.set(n1,v=new Vector<Integer>());
		}
		if (v.size()<=n2)
		{
			v.setSize(n2+1);
		}
		v.set(n2,edgecount);
		phi.set(n1,v);
		edge.set(edgecount++,new Node<TE>(n.getValue(),n1,n2,java.awt.Color.black));
	}
}
